﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace TweenLog2CSV
{
    public static class CSVConverter
    {
        /// <summary>
        /// TweenのログをMicrosoft Excel の CSV 形式に変換します。
        /// </summary>
        /// <param name="log">Tweenのログ</param>
        /// <returns>Excel CSV形式のデータ</returns>
        public static string[] ToCSV(string[] log)
        {
            List<string> list = new List<string>();
            int pos = 0;
            string buf = "";

            // \t08-07-30 20:28:35\t にマッチ
            Regex reg = new Regex(@"\t\d\d\-\d\d\-\d\d \d\d\:\d\d:\d\d\t");
            
            while(pos < log.Length){
                // つぶやきに改行コードが入っていた場合には改行を無視して連結する
                buf += log[pos++];

                // 最初の本文中に\tが含まれていたら半角スペースに置き換える
                // (時刻情報の前の\t区切りは2箇所のみであるべき)
                Match m = reg.Match(buf);
                if (m.Success)
                {
                    int firstTab = buf.IndexOf('\t') + 1;
                    string first = buf.Substring(0, firstTab);
                    string mid = buf.Substring(firstTab, m.Index - firstTab);
                    buf = first + mid.Replace('\t', ' ') + buf.Substring(m.Index);
                }

                string[] elements = buf.Split('\t');

                if (pos < log.Length &&                   // ファイルの最後ではない 
                    (   elements.Length < 7               // \tで分割された要素が7未満
                     || log[pos].Split('\t').Length == 1) // 次の行に\tがない場合
                    )
                {
                    continue;
                }
                else
                {
                    buf = "";
                }

                // フォーマットチェック (4番目の要素は数字のみのはず)
                try
                {
                    Convert.ToInt32(elements[4]);
                }
                catch (Exception)
                {
                    throw new FormatException("Tweenのログではありません\r\n"
                        + string.Join(",", elements));
                }

                // 7より多くの要素がある場合には\tをあらかじめ結合しておく
                for (int i = 7; i < elements.Length; i++)
                {
                    elements[6] += " " + elements[i];
                }
                // Excelの仕様にあわせて , と " を処理する
                for (int i = 0; i < 7; i++)
                {
                    string s = elements[i];
                    if (!s.StartsWith("\""))  // Tween 0.9.0.0 に対応
                    {
                        s = s.Replace("\"", "\"\"");
                        if (s.Contains("\"") || s.Contains(",")) s = "\"" + s + "\"";
                        elements[i] = s;
                    }
                }

                list.Add(string.Join(",", elements, 0, 7));
            }
            return list.ToArray();
        }
    }
}
